# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['jupyter_ascending_patched',
 'jupyter_ascending_patched.handlers',
 'jupyter_ascending_patched.nbextension',
 'jupyter_ascending_patched.notebook',
 'jupyter_ascending_patched.requests',
 'jupyter_ascending_patched.scripts',
 'jupyter_ascending_patched.tests']

package_data = \
{'': ['*'],
 'jupyter_ascending_patched': ['labextension/*'],
 'jupyter_ascending_patched.nbextension': ['static/*']}

install_requires = \
['attrs>=19.0.0',
 'edlib>=1.3.8,<2.0.0',
 'ipywidgets>=7.5.1,<8.0.0',
 'jsonrpcclient[requests]>=3.3.5,<4.0.0',
 'jsonrpcserver[requests]>=4.1.2,<5.0.0',
 'jupytext>=1.3.4,<2.0.0',
 'loguru>=0.4.1',
 'toml>=0.10.0']

setup_kwargs = {
    'name': 'jupyter-ascending-patched',
    'version': '0.1.23.1',
    'description': '',
    'long_description': None,
    'author': 'Josh Albrecht',
    'author_email': 'joshalbrecht@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
