#!/usr/bin/env python
# coding: utf-8

# Copyright (c) tjdevries.
# Distributed under the terms of the Modified BSD License.

# NOTE: Must keep in sync w/ pyproject.toml to publish
version_info = (0, 1, 23, 1)
__version__ = ".".join(map(str, version_info))
